library signature_box;

import 'dart:async';
import 'dart:collection';

import 'package:flutter/material.dart';

class SignatureBox extends StatefulWidget {
  const SignatureBox({
    @required this.width,
    @required this.height,
    @required this.onChanged,
    this.color = Colors.black,
  });

  final double width;
  final double height;
  final Color color;
  final ValueChanged<List<Offset>> onChanged;

  @override
  _SignatureBoxState createState() => _SignatureBoxState();
}

class _SignatureBoxState extends State<SignatureBox> {
  final _points = <Offset>[];

  bool get _hasPoints => _points.isNotEmpty;

  Timer _onChangedTimer;

  @override
  Widget build(BuildContext context) {
    final gestureDetector = GestureDetector(
      onPanUpdate: (details) {
        final RenderBox referenceBox = context.findRenderObject();
        final containerSize = referenceBox.size;
        final localPosition = referenceBox.globalToLocal(details.globalPosition);
        if (localPosition.dx > 0 &&
            localPosition.dy > 0 &&
            localPosition.dx < containerSize.width - 1 &&
            localPosition.dy < containerSize.height - 1) {
          setState(() => _points.add(localPosition));

          // Cancel any existing timer and schedule another. This ensures we don't spam onChanged calls.
          _onChangedTimer?.cancel();
          _onChangedTimer = Timer(
            const Duration(milliseconds: 100),
            () => widget.onChanged(UnmodifiableListView(_points)),
          );
        }
      },
      onPanEnd: (details) => _points.add(null),
      child: CustomPaint(
        painter: _Painter(_points, widget.color),
      ),
    );
    final signatureArea = Container(
      height: widget.height,
      width: double.maxFinite,
      child: gestureDetector,
    );
    final divider = Padding(
      child: Divider(
        height: 2,
        thickness: 1,
        color: Colors.black,
      ),
      padding: EdgeInsets.only(
        bottom: 40,
        left: 20,
        right: 20,
      ),
    );
    final clear = Align(
      alignment: Alignment.bottomRight,
      child: Padding(
        padding: EdgeInsets.all(10),
        child: GestureDetector(
          child: Text('Clear'),
          onTap: () {
            setState(() {
              _points.clear();
              widget.onChanged(UnmodifiableListView(_points));
            });
          },
        ),
      ),
    );
    final layout = Stack(
      children: [
        divider,
        signatureArea,
        if (_hasPoints) clear,
      ],
      alignment: Alignment.bottomCenter,
    );
    final child = Container(
      decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.circular(10.0),
        border: Border.all(color: Colors.blueGrey),
      ),
      child: layout,
    );
    return child;
  }
}

class _Painter extends CustomPainter {
  _Painter(
    this.points,
    this.color,
  );

  final List<Offset> points;
  final Color color;

  @override
  void paint(Canvas canvas, Size size) {
    if (points.length < 2) {
      return;
    }

    final paint = Paint()
      ..color = color
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 3.0;

    for (var i = 0; i < points.length - 1; i++) {
      if (points[i] != null && points[i + 1] != null) {
        canvas.drawLine(points[i], points[i + 1], paint);
      }
    }
  }

  @override
  bool shouldRepaint(_Painter oldDelegate) => true;
}
